@@ -113,6 +113,11 @@ var MALLOC = "dlmalloc";
 // returning NULL (0) when it fails.
 var ABORTING_MALLOC = 1;
 
+// If 1, generated a version of memcpy() and memset() that unroll their
+// copy sizes. If 0, optimizes for size instead to generate a smaller memcpy.
+// This flag only has effect when targeting asm.js.
+var FAST_UNROLLED_MEMCPY_AND_MEMSET = 1;
+
 // If false, we abort with an error if we try to allocate more memory than
 // we can (TOTAL_MEMORY). If true, we will grow the memory arrays at
 // runtime, seamlessly and dynamically. This has a performance cost in asm.js,
@@ -131,6 +136,10 @@ var ABORTING_MALLOC = 1;
 // memory from the system as necessary.
 var ALLOW_MEMORY_GROWTH = 0;
 
+// If true, allows more functions to be added to the table at runtime. This is
+// necessary for dynamic linking, and set automatically in that mode.
+var ALLOW_TABLE_GROWTH = 0;
+
 // where global data begins; the start of static memory. -1 means use the
 // default, any other value will be used as an override
 var GLOBAL_BASE = -1;
@@ -175,14 +184,6 @@ var FORCE_ALIGNED_MEMORY = 0;
 // wasm.
 var WARN_UNALIGNED = 0;
 
-// If enabled, i64 addition etc. is emulated - which is slow but precise. If
-// disabled, we use the 'double trick' which is fast but incurs rounding at high
-// values.  If set to 2, we always include the i64 math code, which is necessary
-// in the case that we can't know at compile time that 64-bit math is needed.
-// For example, if you print 64-bit values with printf, but never add them, we
-// can't know at compile time and you need to set this to 2.
-var PRECISE_I64_MATH = 1;
-
 // 0: Use JS numbers for floating-point values. These are 64-bit and do not model C++
 //    floats exactly, which are 32-bit.
 // 1: Model C++ floats precisely, using Math.fround, polyfilling when necessary. This
@@ -605,12 +606,6 @@ var FS_LOG = 0;
 // case-sensitive, like on Linux.
 var CASE_INSENSITIVE_FS = 0;
 
-// If set to nonzero, MEMFS will always utilize typed arrays as the backing
-// store for appending data to files. The default behavior is to use typed
-// arrays for files when the file size doesn't change after initial creation,
-// and for files that do change size, use normal JS arrays instead.
-var MEMFS_APPEND_TO_TYPED_ARRAYS = 0;
-
 // If set to 0, does not build in any filesystem support. Useful if you are just
 // doing pure computation, but not reading files or using any streams (including
 // fprintf, and other stdio.h things) or anything related. The one exception is
@@ -841,6 +836,14 @@ var DETERMINISTIC = 0;
 // the module. (This allows, in particular, for you to create multiple
 // instantiations, etc.)
 //
+// The default .html shell file provided in MINIMAL_RUNTIME mode shows
+// an example to how the module is instantiated from within the html file.
+// The default .html shell file provided by traditional runtime mode is only
+// compatible with MODULARIZE=0 mode, so when building with traditional
+// runtime, you should provided your own html shell file to perform the
+// instantiation when building with MODULARIZE=1. (For more details, see
+// https://github.com/emscripten-core/emscripten/issues/7950)
+//
 // If you add --pre-js or --post-js files, they will be included inside
 // the module with the rest of the emitted code. That way, they can be
 // optimized together with it. (If you want something outside of the module,
@@ -878,6 +881,16 @@ var MODULARIZE = 0;
 // (since you aren't creating the instance yourself).
 var MODULARIZE_INSTANCE = 0;
 
+// If we separate out asm.js with the --separate-asm option,
+// this is the name of the variable where the generated asm.js
+// Module is assigned to. This name can either be a property
+// of Module, or a freestanding variable name, like "var asmJs".
+// If you are XHRing in multiple asm.js built files, use this option to
+// assign the generated asm.js modules to different variable names
+// so that they do not conflict. Default name is 'Module["asm"]' if a custom
+// name is not passed in.
+var SEPARATE_ASM_MODULE_NAME = '';
+
 // Export using an ES6 Module export rather than a UMD export.  MODULARIZE must
 // be enabled for ES6 exports.
 var EXPORT_ES6 = 0;
@@ -1024,7 +1037,7 @@ var WASM_BACKEND = 0;
 
 // Whether to compile object files as wasm as opposed to the default
 // of using LLVM IR.
-var WASM_OBJECT_FILES = 0;
+var WASM_OBJECT_FILES = 1;
 
 // An optional comma-separated list of script hooks to run after binaryen,
 // in binaryen's /scripts dir.
@@ -1069,6 +1082,16 @@ var BINARYEN_ASYNC_COMPILATION = 1;
 // you can set it to override if you are a Binaryen developer.
 var BINARYEN_ROOT = "";
 
+// WebAssembly defines a "producers section" which compilers and tools can
+// annotate themselves in. Emscripten does not emit this by default, as it
+// increases code size, and some users may not want information about their tools
+// to be included in their builds for privacy or security reasons, see
+// https://github.com/WebAssembly/tool-conventions/issues/93.
+// TODO: currently this flag just controls whether we run the binaryen pass
+//       to strip it out from the wasm (where the LLVM wasm backend may have
+//       created it)
+var EMIT_PRODUCERS_SECTION = 0;
+
 // Whether to legalize the JS FFI interfaces (imports/exports) by wrapping them
 // to automatically demote i64 to i32 and promote f32 to f64. This is necessary
 // in order to interface with JavaScript, both for asm.js and wasm.  For
@@ -1279,6 +1302,10 @@ var FETCH_DEBUG = 0;
 // If nonzero, enables emscripten_fetch API.
 var FETCH = 0;
 
+// Internal: name of the file containing the Fetch *.fetch.js, if relevant
+// Do not set yourself.
+var FETCH_WORKER_FILE = '';
+
 // If set to 1, uses the multithreaded filesystem that is implemented within the
 // asm.js module, using emscripten_fetch. Implies -s FETCH=1.
 var ASMFS = 0;
@@ -1345,6 +1372,9 @@ var MINIFY_ASMJS_IMPORT_NAMES = 0;
 // to JS static allocations
 var STATIC_BUMP = -1;
 
+// the total initial wasm table size.
+var WASM_TABLE_SIZE = 0;
+
 // if set to 1, then generated WASM files will contain a custom
 // "emscripten_metadata" section that contains information necessary
 // to execute the file without the accompanying JS file.
@@ -1357,10 +1387,79 @@ var EMIT_EMSCRIPTEN_METADATA = 0;
 // for effective code size optimizations to take place.
 var SUPPORT_ERRNO = 1;
 
-// Internal: points to a file that lists all asm.js/wasm module exports, annotated
-// with suppressions for Closure compiler, that can be passed as an --externs file
-// to Closure.
+// Internal: An array of all symbols exported from asm.js/wasm module.
 var MODULE_EXPORTS = [];
 
 // Internal (testing only): Disables the blitOffscreenFramebuffer VAO path.
 var OFFSCREEN_FRAMEBUFFER_FORBID_VAO_PATH = 0;
+
+// Internal (testing only): Forces memory growing to fail.
+var TEST_MEMORY_GROWTH_FAILS = 0;
+
+// Advanced: Customize this array to reduce the set of asm.js runtime variables
+// that are generated. This allows minifying extra bit of asm.js code from unused
+// runtime code, if you know some of these are not needed.
+// (think of this as advanced manual DCE)
+var ASM_PRIMITIVE_VARS = ['__THREW__', 'threwValue', 'setjmpId', 'tempInt', 'tempBigInt', 'tempBigIntS', 'tempValue', 'tempDouble', 'tempFloat', 'tempDoublePtr', 'STACKTOP', 'STACK_MAX']
+
+// If true, uses minimal sized runtime without POSIX features, Module, preRun/preInit/etc.,
+// Emscripten built-in XHR loading or library_browser.js. Enable this setting to target
+// the smallest code size possible.
+// Set MINIMAL_RUNTIME=2 to further enable even more code size optimizations. These opts are
+// quite hacky, and work around limitations in Closure and other parts of the build system, so
+// they may not work in all generated programs (But can be useful for really small programs)
+var MINIMAL_RUNTIME = 0;
+
+// If building with MINIMAL_RUNTIME=1 and application uses sbrk()/malloc(), enable this. If you
+// are not using dynamic allocations, can set this to 0 to save code size. This setting is
+// ignored when building with -s MINIMAL_RUNTIME=0.
+var USES_DYNAMIC_ALLOC = 1;
+
+// Advanced manual dead code elimination:
+// Specifies the set of runtime JS functions that should be imported to the asm.js/wasm module.
+// Remove elements from this list to make build smaller if some of these are not needed.
+// In Wasm -O3/-Os builds, adjusting this is not necessary, as the Meta-DCE pass is able to
+// remove these, but if you are targeting asm.js or doing a -O2 build or lower, then this can
+// be beneficial.
+var RUNTIME_FUNCS_TO_IMPORT = ['abort', 'setTempRet0', 'getTempRet0']
+
+// Internal: stores the base name of the output file (-o TARGET_BASENAME.js)
+var TARGET_BASENAME = '';
+
+// If true, compiler supports setjmp() and longjmp(). If false, these APIs are not available.
+// If you are using C++ exceptions, but do not need setjmp()+longjmp() API, then you can set
+// this to 0 to save a little bit of code size and performance when catching exceptions.
+var SUPPORT_LONGJMP = 1;
+
+// If set to 1, disables old deprecated HTML5 API event target lookup behavior. When enabled,
+// there is no "Module.canvas" object, no magic "null" default handling, and DOM element
+// 'target' parameters are taken to refer to CSS selectors, instead of referring to DOM IDs.
+var DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR = 0;
+
+// Specifies whether the generated .html file is run through html-minifier. The set of
+// optimization passes run by html-minifier depends on debug and optimization levels. In
+// -g2 and higher, no minification is performed. In -g1, minification is done, but whitespace
+// is retained. Minification requires at least -O1 or -Os to be used. Pass -s MINIFY_HTML=0
+// to explicitly choose to disable HTML minification altogether.
+var MINIFY_HTML = 1;
+
+// Legacy settings that have been removed, and the values they are now fixed to. These can no
+// longer be changed:
+// [OPTION_NAME, POSSIBLE_VALUES, ERROR_EXPLANATION], where POSSIBLE_VALUES is an array of values that will
+// still be silently accepted by the compiler. First element in the list is the canonical/fixed value going forward.
+// This allows existing build systems to keep specifying one of the supported settings, for backwards compatibility.
+var LEGACY_SETTINGS = [
+  ['ASM_JS', [1, 2], 'ASM_JS must be enabled in fastcomp'],
+  ['SAFE_HEAP', [0, 1], 'safe heap must be 0 or 1 in fastcomp'],
+  ['UNALIGNED_MEMORY', [0], 'forced unaligned memory not supported in fastcomp'],
+  ['FORCE_ALIGNED_MEMORY', [0], 'forced aligned memory is not supported in fastcomp'],
+  ['PGO', [0], 'pgo not supported in fastcomp'],
+  ['QUANTUM_SIZE', [4], 'altering the QUANTUM_SIZE is not supported'],
+  ['FUNCTION_POINTER_ALIGNMENT', [2], 'Starting from Emscripten 1.37.29, no longer available (https://github.com/emscripten-core/emscripten/pull/6091)'],
+  ['BUILD_AS_SHARED_LIB', [0], 'Starting from Emscripten 1.38.16, no longer available (https://github.com/emscripten-core/emscripten/pull/7433)'],
+  ['SAFE_SPLIT_MEMORY', [0], 'Starting from Emscripten 1.38.19, SAFE_SPLIT_MEMORY codegen is no longer available (https://github.com/emscripten-core/emscripten/pull/7465)'],
+  ['SPLIT_MEMORY', [0], 'Starting from Emscripten 1.38.19, SPLIT_MEMORY codegen is no longer available (https://github.com/emscripten-core/emscripten/pull/7465)'],
+  ['BINARYEN_METHOD', ['native-wasm'], 'Starting from Emscripten 1.38.23, Emscripten now always builds either to Wasm (-s WASM=1 - default), or to asm.js (-s WASM=0), other methods are not supported (https://github.com/emscripten-core/emscripten/pull/7836)'],
+  ['PRECISE_I64_MATH', [1, 2], 'Starting from Emscripten 1.38.26, PRECISE_I64_MATH is always enabled (https://github.com/emscripten-core/emscripten/pull/7935)'],
+  ['MEMFS_APPEND_TO_TYPED_ARRAYS', [1], 'Starting from Emscripten 1.38.26, MEMFS_APPEND_TO_TYPED_ARRAYS=0 is no longer supported. MEMFS no longer supports using JS arrays for file data (https://github.com/emscripten-core/emscripten/pull/7918)'],
+];
